# coding: utf-8
"""
    Kubernetes

    No description provided (generated by Swagger Codegen
    https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: v1.14.4

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import urllib3

import copy
import logging
import multiprocessing
import sys

from six import iteritems
from six import with_metaclass
from six.moves import http_client as httplib


class TypeWithDefault(type):

  def __init__(cls, name, bases, dct):
    super(TypeWithDefault, cls).__init__(name, bases, dct)
    cls._default = None

  def __call__(cls):
    if cls._default == None:
      cls._default = type.__call__(cls)
    return copy.copy(cls._default)

  def set_default(cls, default):
    cls._default = copy.copy(default)


class Configuration(with_metaclass(TypeWithDefault, object)):
  """
    NOTE: This class is auto generated by the swagger code generator program.
    Ref: https://github.com/swagger-api/swagger-codegen
    Do not edit the class manually.
    """

  def __init__(self):
    """
        Constructor
        """
    # Default Base url
    self.host = 'https://localhost'
    # Temp file folder for downloading files
    self.temp_folder_path = None

    # Authentication Settings
    # dict to store API key(s)
    self.api_key = {}
    # dict to store API prefix (e.g. Bearer)
    self.api_key_prefix = {}
    # Username for HTTP basic authentication
    self.username = ''
    # Password for HTTP basic authentication
    self.password = ''

    # Logging Settings
    self.logger = {}
    self.logger['package_logger'] = logging.getLogger('client')
    self.logger['urllib3_logger'] = logging.getLogger('urllib3')
    # Log format
    self.logger_format = '%(asctime)s %(levelname)s %(message)s'
    # Log stream handler
    self.logger_stream_handler = None
    # Log file handler
    self.logger_file_handler = None
    # Debug file location
    self.logger_file = None
    # Debug switch
    self.debug = False

    # SSL/TLS verification
    # Set this to false to skip verifying SSL certificate when calling API from https server.
    self.verify_ssl = True
    # Set this to customize the certificate file to verify the peer.
    self.ssl_ca_cert = None
    # client certificate file
    self.cert_file = None
    # client key file
    self.key_file = None
    # Set this to True/False to enable/disable SSL hostname verification.
    self.assert_hostname = None

    # urllib3 connection pool's maximum number of connections saved
    # per pool. urllib3 uses 1 connection as default value, but this is
    # not the best value when you are making a lot of possibly parallel
    # requests to the same host, which is often the case here.
    # cpu_count * 5 is used as default value to increase performance.
    self.connection_pool_maxsize = multiprocessing.cpu_count() * 5

    # Proxy URL
    self.proxy = None
    # Safe chars for path_param
    self.safe_chars_for_path_param = ''

  @property
  def logger_file(self):
    """
        Gets the logger_file.
        """
    return self.__logger_file

  @logger_file.setter
  def logger_file(self, value):
    """
        Sets the logger_file.

        If the logger_file is None, then add stream handler and remove file
        handler.
        Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
    self.__logger_file = value
    if self.__logger_file:
      # If set logging file,
      # then add file handler and remove stream handler.
      self.logger_file_handler = logging.FileHandler(self.__logger_file)
      self.logger_file_handler.setFormatter(self.logger_formatter)
      for _, logger in iteritems(self.logger):
        logger.addHandler(self.logger_file_handler)
        if self.logger_stream_handler:
          logger.removeHandler(self.logger_stream_handler)
    else:
      # If not set logging file,
      # then add stream handler and remove file handler.
      self.logger_stream_handler = logging.StreamHandler()
      self.logger_stream_handler.setFormatter(self.logger_formatter)
      for _, logger in iteritems(self.logger):
        logger.addHandler(self.logger_stream_handler)
        if self.logger_file_handler:
          logger.removeHandler(self.logger_file_handler)

  @property
  def debug(self):
    """
        Gets the debug status.
        """
    return self.__debug

  @debug.setter
  def debug(self, value):
    """
        Sets the debug status.

        :param value: The debug status, True or False.
        :type: bool
        """
    self.__debug = value
    if self.__debug:
      # if debug status is True, turn on debug logging
      for _, logger in iteritems(self.logger):
        logger.setLevel(logging.DEBUG)
      # turn on httplib debug
      httplib.HTTPConnection.debuglevel = 1
    else:
      # if debug status is False, turn off debug logging,
      # setting log level to default `logging.WARNING`
      for _, logger in iteritems(self.logger):
        logger.setLevel(logging.WARNING)
      # turn off httplib debug
      httplib.HTTPConnection.debuglevel = 0

  @property
  def logger_format(self):
    """
        Gets the logger_format.
        """
    return self.__logger_format

  @logger_format.setter
  def logger_format(self, value):
    """
        Sets the logger_format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
    self.__logger_format = value
    self.logger_formatter = logging.Formatter(self.__logger_format)

  def get_api_key_with_prefix(self, identifier):
    """
        Gets API key (with prefix if set).

        :param identifier: The identifier of apiKey.
        :return: The token for api key authentication.
        """
    if self.api_key.get(identifier) and self.api_key_prefix.get(identifier):
      return self.api_key_prefix[identifier] + ' ' + self.api_key[identifier]
    elif self.api_key.get(identifier):
      return self.api_key[identifier]

  def get_basic_auth_token(self):
    """
        Gets HTTP basic authentication header (string).

        :return: The token for basic HTTP authentication.
        """
    return urllib3.util.make_headers(basic_auth=self.username + ':' + self.password)\
                       .get('authorization')

  def auth_settings(self):
    """
        Gets Auth Settings dict for api client.

        :return: The Auth Settings information dict.
        """
    return {
        'BearerToken': {
            'type': 'api_key',
            'in': 'header',
            'key': 'authorization',
            'value': self.get_api_key_with_prefix('authorization')
        },
    }

  def to_debug_report(self):
    """
        Gets the essential information for debugging.

        :return: The report for debugging.
        """
    return 'Python SDK Debug Report:\n'\
           'OS: {env}\n'\
           'Python Version: {pyversion}\n'\
           'Version of the API: v1.14.4\n'\
           'SDK Package Version: 10.0.0-snapshot'.\
           format(env=sys.platform, pyversion=sys.version)



"""Generated message classes for cloudasset version v1p7beta1.

The cloud asset API manages the history and inventory of cloud resources.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'cloudasset'


class BigQueryDestination(_messages.Message):
  r"""A BigQuery destination for exporting assets to.

  Fields:
    dataset: Required. The BigQuery dataset in format
      "projects/projectId/datasets/datasetId", to which the snapshot result
      should be exported. If this dataset does not exist, the export call
      returns an INVALID_ARGUMENT error.
    force: If the destination table already exists and this flag is `TRUE`,
      the table will be overwritten by the contents of assets snapshot. If the
      flag is `FALSE` or unset and the destination table already exists, the
      export call returns an INVALID_ARGUMEMT error.
    partitionSpec: [partition_spec] determines whether to export to
      partitioned table(s) and how to partition the data. If [partition_spec]
      is unset or [partition_spec.partition_key] is unset or
      `PARTITION_KEY_UNSPECIFIED`, the snapshot results will be exported to
      non-partitioned table(s). [force] will decide whether to overwrite
      existing table(s). If [partition_spec] is specified. First, the snapshot
      results will be written to partitioned table(s) with two additional
      timestamp columns, readTime and requestTime, one of which will be the
      partition key. Secondly, in the case when any destination table already
      exists, it will first try to update existing table's schema as necessary
      by appending additional columns. Then, if [force] is `TRUE`, the
      corresponding partition will be overwritten by the snapshot results
      (data in different partitions will remain intact); if [force] is unset
      or `FALSE`, it will append the data. An error will be returned if the
      schema update or data appension fails.
    separateTablesPerAssetType: If this flag is `TRUE`, the snapshot results
      will be written to one or multiple tables, each of which contains
      results of one asset type. The [force] and [partition_spec] fields will
      apply to each of them. Field [table] will be concatenated with "_" and
      the asset type names (see https://cloud.google.com/asset-
      inventory/docs/supported-asset-types for supported asset types) to
      construct per-asset-type table names, in which all non-alphanumeric
      characters like "." and "/" will be substituted by "_". Example: if
      field [table] is "mytable" and snapshot results contain
      "storage.googleapis.com/Bucket" assets, the corresponding table name
      will be "mytable_storage_googleapis_com_Bucket". If any of these tables
      does not exist, a new table with the concatenated name will be created.
      When [content_type] in the ExportAssetsRequest is `RESOURCE`, the schema
      of each table will include RECORD-type columns mapped to the nested
      fields in the Asset.resource.data field of that asset type (up to the 15
      nested level BigQuery supports
      (https://cloud.google.com/bigquery/docs/nested-repeated#limitations)).
      The fields in >15 nested levels will be stored in JSON format string as
      a child column of its parent RECORD column. If error occurs when
      exporting to any table, the whole export call will return an error but
      the export results that already succeed will persist. Example: if
      exporting to table_type_A succeeds when exporting to table_type_B fails
      during one export call, the results in table_type_A will persist and
      there will not be partial results persisting in a table.
    table: Required. The BigQuery table to which the snapshot result should be
      written. If this table does not exist, a new table with the given name
      will be created.
  """

  dataset = _messages.StringField(1)
  force = _messages.BooleanField(2)
  partitionSpec = _messages.MessageField('PartitionSpec', 3)
  separateTablesPerAssetType = _messages.BooleanField(4)
  table = _messages.StringField(5)


class CloudassetExportAssetsRequest(_messages.Message):
  r"""A CloudassetExportAssetsRequest object.

  Fields:
    exportAssetsRequest: A ExportAssetsRequest resource to be passed as the
      request body.
    parent: Required. The relative name of the root asset. This can only be an
      organization number (such as "organizations/123"), a project ID (such as
      "projects/my-project-id"), or a project number (such as
      "projects/12345"), or a folder number (such as "folders/123").
  """

  exportAssetsRequest = _messages.MessageField('ExportAssetsRequest', 1)
  parent = _messages.StringField(2, required=True)


class ExportAssetsRequest(_messages.Message):
  r"""Export asset request.

  Enums:
    ContentTypeValueValuesEnum: Asset content type. If not specified, no
      content but the asset name will be returned.

  Fields:
    assetTypes: A list of asset types to take a snapshot for. For example:
      "compute.googleapis.com/Disk". Regular expressions are also supported.
      For example: * "compute.googleapis.com.*" snapshots resources whose
      asset type starts with "compute.googleapis.com". * ".*Instance"
      snapshots resources whose asset type ends with "Instance". *
      ".*Instance.*" snapshots resources whose asset type contains "Instance".
      See [RE2](https://github.com/google/re2/wiki/Syntax) for all supported
      regular expression syntax. If the regular expression does not match any
      supported asset type, an INVALID_ARGUMENT error will be returned. If
      specified, only matching assets will be returned, otherwise, it will
      snapshot all asset types. See [Introduction to Cloud Asset
      Inventory](https://cloud.google.com/asset-inventory/docs/overview) for
      all supported asset types.
    contentType: Asset content type. If not specified, no content but the
      asset name will be returned.
    outputConfig: Required. Output configuration indicating where the results
      will be output to.
    readTime: Timestamp to take an asset snapshot. This can only be set to a
      timestamp between the current time and the current time minus 35 days
      (inclusive). If not specified, the current time will be used. Due to
      delays in resource data collection and indexing, there is a volatile
      window during which running the same query may get different results.
    relationshipTypes: A list of relationship types to export, for example:
      `INSTANCE_TO_INSTANCEGROUP`. This field should only be specified if
      content_type=RELATIONSHIP. If specified, it will snapshot [asset_types]'
      specified relationships, or give errors if any relationship_types'
      supported types are not in [asset_types]. If not specified, it will
      snapshot all [asset_types]' supported relationships. An unspecified
      [asset_types] field means all supported asset_types. See [Introduction
      to Cloud Asset Inventory](https://cloud.google.com/asset-
      inventory/docs/overview) for all supported asset types and relationship
      types.
  """

  class ContentTypeValueValuesEnum(_messages.Enum):
    r"""Asset content type. If not specified, no content but the asset name
    will be returned.

    Values:
      CONTENT_TYPE_UNSPECIFIED: Unspecified content type.
      RESOURCE: Resource metadata.
      IAM_POLICY: The actual IAM policy set on a resource.
      ORG_POLICY: The Cloud Organization Policy set on an asset.
      ACCESS_POLICY: The Cloud Access context manager Policy set on an asset.
      RELATIONSHIP: The related resources.
    """
    CONTENT_TYPE_UNSPECIFIED = 0
    RESOURCE = 1
    IAM_POLICY = 2
    ORG_POLICY = 3
    ACCESS_POLICY = 4
    RELATIONSHIP = 5

  assetTypes = _messages.StringField(1, repeated=True)
  contentType = _messages.EnumField('ContentTypeValueValuesEnum', 2)
  outputConfig = _messages.MessageField('OutputConfig', 3)
  readTime = _messages.StringField(4)
  relationshipTypes = _messages.StringField(5, repeated=True)


class GcsDestination(_messages.Message):
  r"""A Cloud Storage location.

  Fields:
    uri: The uri of the Cloud Storage object. It's the same uri that is used
      by gsutil. Example: "gs://bucket_name/object_name". See [Viewing and
      Editing Object Metadata](https://cloud.google.com/storage/docs/viewing-
      editing-metadata) for more information.
    uriPrefix: The uri prefix of all generated Cloud Storage objects. Example:
      "gs://bucket_name/object_name_prefix". Each object uri is in format:
      "gs://bucket_name/object_name_prefix/{ASSET_TYPE}/{SHARD_NUMBER} and
      only contains assets for that type. starts from 0. Example:
      "gs://bucket_name/object_name_prefix/compute.googleapis.com/Disk/0" is
      the first shard of output objects containing all
      compute.googleapis.com/Disk assets. An INVALID_ARGUMENT error will be
      returned if file with the same name
      "gs://bucket_name/object_name_prefix" already exists.
  """

  uri = _messages.StringField(1)
  uriPrefix = _messages.StringField(2)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OutputConfig(_messages.Message):
  r"""Output configuration for export assets destination.

  Fields:
    bigqueryDestination: Destination on BigQuery. The output table stores the
      fields in asset proto as columns in BigQuery.
    gcsDestination: Destination on Cloud Storage.
  """

  bigqueryDestination = _messages.MessageField('BigQueryDestination', 1)
  gcsDestination = _messages.MessageField('GcsDestination', 2)


class PartitionSpec(_messages.Message):
  r"""Specifications of BigQuery partitioned table as export destination.

  Enums:
    PartitionKeyValueValuesEnum: The partition key for BigQuery partitioned
      table.

  Fields:
    partitionKey: The partition key for BigQuery partitioned table.
  """

  class PartitionKeyValueValuesEnum(_messages.Enum):
    r"""The partition key for BigQuery partitioned table.

    Values:
      PARTITION_KEY_UNSPECIFIED: Unspecified partition key. If used, it means
        using non-partitioned table.
      READ_TIME: The time when the snapshot is taken. If specified as
        partition key, the result table(s) is partitoned by the additional
        timestamp column, readTime. If [read_time] in ExportAssetsRequest is
        specified, the readTime column's value will be the same as it.
        Otherwise, its value will be the current time that is used to take the
        snapshot.
      REQUEST_TIME: The time when the request is received and started to be
        processed. If specified as partition key, the result table(s) is
        partitoned by the requestTime column, an additional timestamp column
        representing when the request was received.
    """
    PARTITION_KEY_UNSPECIFIED = 0
    READ_TIME = 1
    REQUEST_TIME = 2

  partitionKey = _messages.EnumField('PartitionKeyValueValuesEnum', 1)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod, ABCMeta
from typing import AsyncContextManager

from google.cloud.pubsublite_v1 import Cursor


class Committer(AsyncContextManager, metaclass=ABCMeta):
    """
    A Committer is able to commit subscribers' completed offsets.
    """

    @abstractmethod
    def commit(self, cursor: Cursor) -> None:
        """
        Start the commit for a cursor.

        Raises:
          GoogleAPICallError: When the committer terminates in failure.
        """
        pass

    @abstractmethod
    async def wait_until_empty(self):
        """
        Flushes pending commits and waits for all outstanding commit responses from the server.

        Raises:
          GoogleAPICallError: When the committer terminates in failure.
        """
        pass



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import NamedTuple

from google.api_core.exceptions import InvalidArgument


class CloudRegion(NamedTuple):
    name: str

    def __str__(self):
        return self.name

    @staticmethod
    def parse(to_parse: str):
        splits = to_parse.split("-")
        if len(splits) != 2:
            raise InvalidArgument("Invalid region name: " + to_parse)
        return CloudRegion(name=splits[0] + "-" + splits[1])

    @property
    def region(self):
        return self


class CloudZone(NamedTuple):
    region: CloudRegion
    zone_id: str

    def __str__(self):
        return f"{self.region.name}-{self.zone_id}"

    @staticmethod
    def parse(to_parse: str):
        splits = to_parse.split("-")
        if len(splits) != 3 or len(splits[2]) != 1:
            raise InvalidArgument("Invalid zone name: " + to_parse)
        region = CloudRegion(name=splits[0] + "-" + splits[1])
        return CloudZone(region, zone_id=splits[2])



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Reset command for the Org Policy CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import copy

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.org_policies import interfaces

DETAILED_HELP = {
    'DESCRIPTION':
        """\
      Resets the policy to the default for the constraint.
      """,
    'EXAMPLES':
        """\
      To reset the policy associated with the constraint 'gcp.resourceLocations' and
      the Project 'foo-project', run:

        $ {command} gcp.resourceLocations --project=foo-project
      """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Reset(interfaces.OrgPolicyGetAndUpdateCommand):
  """Reset the policy to the default for the constraint."""

  def UpdatePolicy(self, policy, args):
    """Sets the reset field on the policy to True.

    If reset is set to True, no rules can be set on the policy and
    inheritFromParent has to be False. As such, this also deletes all rules on
    the policy and sets inheritFromParent to False.

    Args:
      policy: messages.GoogleCloudOrgpolicyV2alpha1Policy, The policy to be
        updated.
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the Args method.

    Returns:
      The updated policy.
    """
    new_policy = copy.deepcopy(policy)
    new_policy.spec.reset = True
    new_policy.spec.rules = []
    new_policy.spec.inheritFromParent = False

    return new_policy


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ResetALPHA(Reset):
  """Reset the policy to the default for the constraint."""
  pass


Reset.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to query activities."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import sys

from apitools.base.py import list_pager

from googlecloudsdk.api_lib.policy_intelligence import policy_analyzer
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base

_DETAILED_HELP = {
    'brief':
        """Query activities on cloud resource.
        """,
    'DESCRIPTION':
        """\
     Query activities with certain types of specific container resource. For --activity-type, supported values are:
     - serviceAccountLastAuthentication
     - serviceAccountKeyLastAuthentication
        """,
    'EXAMPLES':
        """\
    To query serviceAccountKeyLastAuthentication activities of a project, run:

    $ {command} --activity-type=serviceAccountKeyLastAuthentication --project=project-id

    To query serviceAccountLastAuthentication activities of a project with no limit, run:

    $ {command} --activity-type=serviceAccountLastAuthentication --project=project-id --limit=unlimited

    To query serviceAccountLastAuthentication with filtering on certain service account, run:

    $ {command} --activity-type=serviceAccountLastAuthentication --project=project-id --query-filter='activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name@project-id.iam.gserviceaccount.com"'

    To query serviceAccountLastAuthentication with filtering on multiple service accounts, run:

    $ {command} --activity-type=serviceAccountLastAuthentication --project=project-id --query-filter='activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-1@project-id.iam.gserviceaccount.com" OR activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-2@project-id.iam.gserviceaccount.com" OR activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-3@project-id.iam.gserviceaccount.com"'
        """
}

_DETAILED_HELP_ALPHA = {
    'brief':
        """Query activities on cloud resource.
        """,
    'DESCRIPTION':
        """\
     Query activities with certain types of specific container resource. For --activity-type, supported values are:
     - serviceAccountLastAuthentication
     - serviceAccountKeyLastAuthentication
     - dailyAuthorization
        """,
    'EXAMPLES':
        """\
    To query serviceAccountKeyLastAuthentication activities of a project, run:

    $ {command} --activity-type=serviceAccountKeyLastAuthentication --project=project-id

    To query serviceAccountLastAuthentication activities of a project with no limit, run:

    $ {command} --activity-type=serviceAccountLastAuthentication --project=project-id --limit=unlimited

    To query serviceAccountLastAuthentication with filtering on certain service account, run:

    $ {command} --activity-type=serviceAccountLastAuthentication --project=project-id --query-filter='activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name@project-id.iam.gserviceaccount.com"'

    To query serviceAccountLastAuthentication with filtering on multiple service accounts, run:

    $ {command} --activity-type=serviceAccountLastAuthentication --project=project-id --query-filter='activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-1@project-id.iam.gserviceaccount.com" OR activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-2@project-id.iam.gserviceaccount.com" OR activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-3@project-id.iam.gserviceaccount.com"'

    To query dailyAuthorization activities of a project, run:

    $ {command} --activity-type=dailyAuthorization --project=project-id

    To query dailyAuthorization of a project with filtering on certain resource, permission, principal and date, run:

    $ {command} --activity-type=dailyAuthorization --project=project-id --query-filter='activities.activity.full_resource_name="<full_resource_name>" AND activities.activity.permission="<permission_name>" AND activities.activity.principal="<principal_email>" AND activities.activity.date="<YYYY-MM-DD>"'
    """
}


def _Args(parser):
  """Parses arguments for the commands."""
  parser.add_argument(
      '--activity-type',
      required=True,
      type=str,
      choices=[
          'serviceAccountLastAuthentication',
          'serviceAccountKeyLastAuthentication'
      ],
      help="""Type of the activities.
      """)
  parser.add_mutually_exclusive_group(required=True).add_argument(
      '--project',
      type=str,
      help="""The project ID or number to query the activities.
      """)
  parser.add_argument(
      '--query-filter',
      type=str,
      default='',
      help='Filter on activities, separated by "OR" if multiple filters are specified. At most 10 filter restrictions are supported in the query-filter. e.g. --query-filter=\'activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-1@project-id.iam.gserviceaccount.com" OR activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-2@project-id.iam.gserviceaccount.com"\''
  )
  parser.add_argument(
      '--limit',
      type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
      default=1000,
      help='Max number of query result. Default to be 1000 and max to be unlimited, i.e., --limit=unlimited.'
  )
  parser.add_argument(
      '--page-size',
      type=arg_parsers.BoundedInt(1, 1000),
      default=500,
      help='Max page size for each http response. Default to be 500 and max to be 1000.'
  )


def _ArgsAlpha(parser):
  """Parses arguments for the commands."""
  parser.add_argument(
      '--activity-type',
      required=True,
      type=str,
      choices=[
          'serviceAccountLastAuthentication',
          'serviceAccountKeyLastAuthentication', 'dailyAuthorization'
      ],
      help="""Type of the activities.
      """)
  parser.add_mutually_exclusive_group(required=True).add_argument(
      '--project',
      type=str,
      help="""The project ID or number to query the activities.
      """)
  parser.add_argument(
      '--query-filter',
      type=str,
      default='',
      help="""Filter on activities. \n
      For last authentication activities, this field is separated by "OR" if multiple filters are specified. At most 10 filter restrictions are supported in the query-filter. \n
        e.g. --query-filter=\'activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-1@project-id.iam.gserviceaccount.com" OR activities.full_resource_name="//iam.googleapis.com/projects/project-id/serviceAccounts/service-account-name-2@project-id.iam.gserviceaccount.com"\'\n
      For daily authorization activities, this field is separated by "OR" and "AND". At most 10 filter restrictions per layer and at most 2 layers are supported in the query-filter. \n
        e.g. --query-filter=\'activities.activity.date="2022-01-01" AND activities.activity.permission="spanner.databases.list" AND (activities.activity.principal="principal_1@your-organization.com" OR activities.activity.principal="principal_2@your-organization.com")'"""
  )
  parser.add_argument(
      '--limit',
      type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
      default=1000,
      help='Max number of query result. Default to be 1000 and max to be unlimited, i.e., --limit=unlimited.'
  )
  parser.add_argument(
      '--page-size',
      type=arg_parsers.BoundedInt(1, 1000),
      default=500,
      help='Max page size for each http response. Default to be 500 and max to be 1000.'
  )


def _Run(args):
  policy_analyzer_client, messages = policy_analyzer.GetClientAndMessages()
  query_activity_parent = 'projects/{0}/locations/global/activityTypes/{1}'.format(
      args.project, args.activity_type)
  query_activity_request = messages.PolicyanalyzerProjectsLocationsActivityTypesActivitiesQueryRequest(
      parent=query_activity_parent, filter=args.query_filter)
  policy_analyzer_service = policy_analyzer_client.ProjectsLocationsActivityTypesActivitiesService(
      policy_analyzer_client)
  return list_pager.YieldFromList(
      policy_analyzer_service,
      query_activity_request,
      method='Query',
      batch_size=args.page_size,
      field='activities',
      limit=args.limit,
      batch_size_attribute='pageSize')


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
class QueryActivityAlpha(base.Command):
  """Query activities on cloud resource."""

  detailed_help = _DETAILED_HELP_ALPHA

  @staticmethod
  def Args(parser):
    """Parses arguments for the commands."""
    _ArgsAlpha(parser)

  def Run(self, args):
    return _Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.Hidden
class QueryActivityBeta(base.Command):
  """Query activities on cloud resource."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Parses arguments for the commands."""
    _Args(parser)

  def Run(self, args):
    return _Run(args)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class QueryActivityGA(base.Command):
  """Query activities on cloud resource."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Parses arguments for the commands."""
    _Args(parser)

  def Run(self, args):
    return _Run(args)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List reusable configs."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import constants
from googlecloudsdk.api_lib.privateca import locations
from googlecloudsdk.api_lib.privateca import resource_utils
from googlecloudsdk.api_lib.util import common_args
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import response_utils

# Resource IDs of the currently available reusable configs.
_KnownResourceIds = [
    'leaf-client-tls',
    'leaf-code-signing',
    'leaf-mtls',
    'leaf-server-tls',
    'leaf-smime',
    'root-unconstrained',
    'subordinate-client-tls-pathlen-0',
    'subordinate-code-signing-pathlen-0',
    'subordinate-mtls-pathlen-0',
    'subordinate-server-tls-pathlen-0',
    'subordinate-smime-pathlen-0',
    'subordinate-unconstrained-pathlen-0',
]


def _GetLocation(args):
  if args.IsSpecified('location'):
    return args.location

  # During alpha, we replicate the same set of resources across all locations.
  return locations.GetSupportedLocations()[0]


class List(base.ListCommand):
  """List reusable configs."""

  @staticmethod
  def Args(parser):
    base.Argument(
        '--location',
        help=('Location of the reusable configs. If this is not specified, it '
              'defaults to the first location supported by the service.'
             )).AddToParser(parser)
    base.PAGE_SIZE_FLAG.SetDefault(parser, 100)
    base.SORT_BY_FLAG.SetDefault(parser, 'name')

    parser.display_info.AddFormat("""
        table(
          name.scope("reusableConfigs"):label=NAME,
          name.scope("locations").segment(0):label=LOCATION,
          description)
        """)
    parser.display_info.AddUriFunc(
        resource_utils.MakeGetUriFunc(
            'privateca.projects.locations.reusableConfigs'))

  def ListLatestReusableConfigs(self, args, project, location):
    """Makes one or more List requests for the latest reusable config resources."""
    parent = 'projects/{}/locations/{}'.format(project, location)
    request = self.messages.PrivatecaProjectsLocationsReusableConfigsListRequest(
        parent=parent,
        orderBy=common_args.ParseSortByArg(args.sort_by),
        filter=args.filter)

    return list_pager.YieldFromList(
        self.client.projects_locations_reusableConfigs,
        request,
        field='reusableConfigs',
        limit=args.limit,
        batch_size_attribute='pageSize',
        batch_size=args.page_size,
        get_field_func=response_utils.GetFieldAndLogUnreachable)

  def ListKnownReusableConfigs(self, project, location):
    """Make a series of Get requests for the known reusable config resources."""
    parent = 'projects/{}/locations/{}'.format(project, location)

    for resource_id in _KnownResourceIds:
      resource_name = '{}/reusableConfigs/{}'.format(parent, resource_id)
      yield self.client.projects_locations_reusableConfigs.Get(
          self.messages.PrivatecaProjectsLocationsReusableConfigsGetRequest(
              name=resource_name))

  def Run(self, args):
    """Runs the command."""
    self.client = privateca_base.GetClientInstance()
    self.messages = privateca_base.GetMessagesModule()

    project = constants.PREDEFINED_REUSABLE_CONFIG_PROJECT
    location = _GetLocation(args)

    # TODO(b/170409946): Revert to ListLatestReusableConfigs after IAM issue.
    return self.ListKnownReusableConfigs(project, location)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for list TPU node and GCE VM combinations created."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags
from googlecloudsdk.command_lib.compute.tpus.execution_groups import util as tpu_utils


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.ALPHA)
class List(base.ListCommand):
  r"""List TPU Node+VM groups created by execution group.

  ## EXAMPLES

  To list all active execution groups, run:

    $ {command} --zone=test-zone --project=test-project
  """

  @classmethod
  def Args(cls, parser):
    flags.AddZoneFlag(parser, resource_type='tpu', operation_type='list')
    parser.display_info.AddFormat('table(name, status)')

  def Run(self, args):
    tpu_utils.DefaultArgs.ValidateZone(args)

    responses = []
    instances = {}
    nodes = {}
    instance_helper = tpu_utils.Instance(self.ReleaseTrack())
    for instance in instance_helper.List(args.zone):
      instances[instance.name] = instance

    node_helper = tpu_utils.TPUNode(self.ReleaseTrack())
    for node in node_helper.List(args.zone):
      nodes[node_helper.NodeName(node)] = node

    for name, instance in instances.items():
      if name not in nodes.keys():
        responses.append(ListResult(name, 'Paused'))
      elif instance_helper.IsRunning(instance) and node_helper.IsRunning(
          nodes[name]):
        responses.append(ListResult(name, 'Running'))
      else:
        responses.append(ListResult(name, 'Unknown Status'))
    return sorted(responses)


class ListResult(object):

  def __init__(self, name, status):
    self.name = name
    self.status = status

  def __gt__(self, lr):
    return self.name + self.status > lr.name + lr.status



# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This package provides compatibility interfaces for OCI."""

from __future__ import absolute_import
from __future__ import division

from __future__ import print_function

import json

from containerregistry.client.v2_2 import docker_http
from containerregistry.client.v2_2 import docker_image
from containerregistry.client.v2_2 import docker_image_list



class OCIFromV22(docker_image.Delegate):
  """This compatibility interface serves an OCI image from a v2_2 image."""

  def manifest(self):
    """Override."""
    manifest = json.loads(self._image.manifest())

    manifest['mediaType'] = docker_http.OCI_MANIFEST_MIME
    manifest['config']['mediaType'] = docker_http.OCI_CONFIG_JSON_MIME
    for layer in manifest['layers']:
      layer['mediaType'] = docker_http.OCI_LAYER_MIME

    return json.dumps(manifest, sort_keys=True)

  def media_type(self):
    """Override."""
    return docker_http.OCI_MANIFEST_MIME

  def __enter__(self):
    return self

  def __exit__(self, unused_type, unused_value, unused_traceback):
    """Override."""
    pass


class V22FromOCI(docker_image.Delegate):
  """This compatibility interface serves a v2_2 image from an OCI image."""

  def manifest(self):
    """Override."""
    manifest = json.loads(self._image.manifest())

    manifest['mediaType'] = docker_http.MANIFEST_SCHEMA2_MIME
    manifest['config']['mediaType'] = docker_http.CONFIG_JSON_MIME
    for layer in manifest['layers']:
      layer['mediaType'] = docker_http.LAYER_MIME

    return json.dumps(manifest, sort_keys=True)

  def media_type(self):
    """Override."""
    return docker_http.MANIFEST_SCHEMA2_MIME

  def __enter__(self):
    return self

  def __exit__(self, unused_type, unused_value, unused_traceback):
    """Override."""
    pass


class IndexFromList(docker_image_list.Delegate):
  """This compatibility interface serves an Image Index from a Manifest List."""

  def __init__(self,
               image,
               recursive = True):
    """Constructor.

    Args:
      image: a DockerImageList on which __enter__ has already been called.
      recursive: whether to recursively convert child manifests to OCI types.
    """
    super(IndexFromList, self).__init__(image)
    self._recursive = recursive

  def manifest(self):
    """Override."""
    manifest = json.loads(self._image.manifest())
    manifest['mediaType'] = docker_http.OCI_IMAGE_INDEX_MIME
    return json.dumps(manifest, sort_keys=True)

  def media_type(self):
    """Override."""
    return docker_http.OCI_IMAGE_INDEX_MIME

  def __enter__(self):
    if not self._recursive:
      return self

    converted = []
    for platform, child in self._image:
      if isinstance(child, docker_image_list.DockerImageList):
        with IndexFromList(child) as index:
          converted.append((platform, index))
      else:
        assert isinstance(child, docker_image.DockerImage)
        with OCIFromV22(child) as oci:
          converted.append((platform, oci))
    with docker_image_list.FromList(converted) as index:
      self._image = index
    return self

  def __exit__(self, unused_type, unused_value, unused_traceback):
    """Override."""
    pass


class ListFromIndex(docker_image_list.Delegate):
  """This compatibility interface serves a Manifest List from an Image Index."""

  def __init__(self,
               image,
               recursive = True):
    """Constructor.

    Args:
      image: a DockerImageList on which __enter__ has already been called.
      recursive: whether to recursively convert child manifests to Docker types.
    """
    super(ListFromIndex, self).__init__(image)
    self._recursive = recursive

  def manifest(self):
    """Override."""
    manifest = json.loads(self._image.manifest())
    manifest['mediaType'] = docker_http.MANIFEST_LIST_MIME
    return json.dumps(manifest, sort_keys=True)

  def media_type(self):
    """Override."""
    return docker_http.MANIFEST_LIST_MIME

  def __enter__(self):
    if not self._recursive:
      return self

    converted = []
    for platform, child in self._image:
      if isinstance(child, docker_image_list.DockerImageList):
        with ListFromIndex(child) as image_list:
          converted.append((platform, image_list))
      else:
        assert isinstance(child, docker_image.DockerImage)
        with V22FromOCI(child) as v22:
          converted.append((platform, v22))
    with docker_image_list.FromList(converted) as image_list:
      self._image = image_list
    return self

  def __exit__(self, unused_type, unused_value, unused_traceback):
    """Override."""
    pass



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe a KubeRun application."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kuberun import kuberun_command
from googlecloudsdk.core import log

_DETAILED_HELP = ({
    'EXAMPLES':
        """
        To show all the data about the KubeRun application associated with the
        current working directory, run:

            $ {command}
        """,
})


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Describe(kuberun_command.KubeRunCommand, base.DescribeCommand):
  """Describes a KubeRun application."""

  detailed_help = _DETAILED_HELP
  flags = []

  @classmethod
  def Args(cls, parser):
    super(Describe, cls).Args(parser)

  def Command(self):
    return ['applications', 'describe']

  def SuccessResult(self, out, args):
    # TODO(b/170429098): handle this as JSON
    if not out:
      return out
    return out + '\n'

  # TODO(b/170429098): remove this workaround
  def Display(self, args, output):
    log.out.write(output)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'vmware sddc clusters removenodes' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.vmware.sddc.clusters import ClustersClient
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.vmware.sddc import flags

DETAILED_HELP = {
    'DESCRIPTION':
        """
          Remove a node from the VMware Engine cluster. Successful removal
          of a node results in a cluster in READY state. Check the progress
          of a cluster using `gcloud alpha vmware clusters list`.

          For more examples, refer to the EXAMPLES section below.
        """,
    'EXAMPLES':
        """
          To remove a node from the cluster called ``my-cluster'', run:

            $ {command} my-cluster --privatecloud=my-privatecloud --region=us-central1 --project=my-project

          Or:

            $ {command} my-cluster

          In the second example, the project and region are taken from
          gcloud properties core/project and vmware/region.
    """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class RemoveNodes(base.UpdateCommand):
  """Remove a node from the cluster in a VMware Engine private cloud."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.AddClusterArgToParser(parser)

  def Run(self, args):
    cluster = args.CONCEPTS.cluster.Parse()
    client = ClustersClient()
    operation = client.RemoveNodes(cluster, 1)
    return client.WaitForOperation(
        operation,
        'waiting for node to be removed from the cluster [{}]'.format(cluster))


RemoveNodes.detailed_help = DETAILED_HELP



# Copyright 2017 Google Inc. All Rights Reserved.
#
"""A convenience wrapper for starting Dataflow SQL shell."""

import os

import bootstrapping
from googlecloudsdk.command_lib.util import java
from googlecloudsdk.core.updater import update_manager

# Path to the unpacked component
_COMPONENT_DIR = os.path.join('platform', 'dataflow-sql')

# Path to the directory of unpacked jars relative to the SDK root
_JAR_DIR = os.path.join(_COMPONENT_DIR, 'lib')

# The main jar in the _JAR_DIR, must be first for precedence
_MAIN_JAR = 'dataflow_sql_shell.jar'

# Name of the main class
_CLASSNAME = 'org.apache.beam.sdk.extensions.sql.jdbc.BeamSqlLine'


def main():
  """Launches the Dataflow SQL shell."""
  bootstrapping.CommandStart('dataflow-sql', component_id='dataflow-sql')
  bootstrapping.CheckUpdates('dataflow-sql')
  update_manager.UpdateManager.EnsureInstalledAndRestart(
      ['dataflow-sql'], command=__file__)
  java_bin = java.RequireJavaInstalled('Dataflow SQL')
  bootstrapping.ExecuteJavaClass(
      java_bin,
      jar_dir=_JAR_DIR,
      main_jar=_MAIN_JAR,
      main_class=_CLASSNAME,
      main_args=['-nn', 'DFSQL', '-u', 'jdbc:beam:userAgent=DataflowSQL'])


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for generating API-specific RequestConfig objects.

RequestConfig is provider neutral and should be subclassed into a
provider-specific class (e.g. GcsRequestConfig) by the factory method.

RequestConfig can hold a BucketConfig or ObjectConfig. These classes also
have provider-specific subclasses (e.g. S3ObjectConfig).
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.storage import encryption_util
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.core import log
from googlecloudsdk.core.util import debug_output


DEFAULT_CONTENT_TYPE = 'application/octet-stream'


# Bucket update fields and corresponding flags unsupported by S3.
S3_ERROR_FLAGS = {
    'gzip_in_flight': [
        '--gzip-in-flight-all', '-J', '--gzip-in-flight-extensions', '-j'
    ],
}
S3_WARNING_FLAGS = [
    ('default_encryption_key', '--default-encryption-key'),
    ('default_event_based_hold', '--default-event-based-hold'),
    ('default_storage_class', '--default-storage-class'),
    ('retention_period', '--retention-period'),
    ('uniform_bucket_level_access', '--uniform-bucket-level-access'),
]


class _BucketConfig(object):
  """Holder for generic bucket fields.

  Attributes:
    cors_file_path (None|str): Path to file with CORS settings.
    labels_file_path (None|str): Path to file with labels settings.
    labels_to_append (None|Dict): Labels to add to a bucket.
    labels_to_remove (None|List[str]): Labels to remove from a bucket.
    lifecycle_file_path (None|str): Path to file with lifecycle settings.
    location (str|None): Location of bucket.
    log_bucket (str|None): Destination bucket for current bucket's logs.
    log_object_prefix (str|None): Prefix for objects containing logs.
    requester_pays (bool|None): If set requester pays all costs related to
      accessing the bucket and its objects.
    versioning (None|bool): Whether to turn on object versioning in a bucket.
    web_error_page (None|str): Error page address if bucket is being used
      to host a website.
    web_main_page_suffix (None|str): Suffix of main page address if bucket is
      being used to host a website.
  """

  def __init__(self,
               cors_file_path=None,
               labels_file_path=None,
               labels_to_append=None,
               labels_to_remove=None,
               lifecycle_file_path=None,
               location=None,
               log_bucket=None,
               log_object_prefix=None,
               requester_pays=None,
               versioning=None,
               web_error_page=None,
               web_main_page_suffix=None):
    self.location = location
    self.cors_file_path = cors_file_path
    self.labels_file_path = labels_file_path
    self.labels_to_append = labels_to_append
    self.labels_to_remove = labels_to_remove
    self.lifecycle_file_path = lifecycle_file_path
    self.log_bucket = log_bucket
    self.log_object_prefix = log_object_prefix
    self.requester_pays = requester_pays
    self.versioning = versioning
    self.web_error_page = web_error_page
    self.web_main_page_suffix = web_main_page_suffix

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (super(_BucketConfig, self).__eq__(other) and
            self.cors_file_path == other.cors_file_path and
            self.labels_file_path == other.labels_file_path and
            self.labels_to_append == other.labels_to_append and
            self.labels_to_remove == other.labels_to_remove and
            self.lifecycle_file_path == other.lifecycle_file_path and
            self.location == other.location and
            self.log_bucket == other.log_bucket and
            self.log_object_prefix == other.log_object_prefix and
            self.requester_pays == other.requester_pays and
            self.versioning == other.versioning and
            self.web_error_page == other.web_error_page and
            self.web_main_page_suffix == other.web_main_page_suffix)

  def __repr__(self):
    return debug_output.generic_repr(self)


class _GcsBucketConfig(_BucketConfig):
  """Holder for GCS-specific bucket fields.

  See superclass for remaining attributes.

  Subclass Attributes:
    default_encryption_key (str|None): A key used to encrypt objects
      added to the bucket.
    default_event_based_hold (bool|None): Determines if event-based holds will
      automatically be applied to new objects in bucket.
    default_storage_class (str|None): Storage class assigned to objects in the
      bucket by default.
    retention_period (int|None): Minimum retention period in seconds for objects
      in a bucket. Attempts to delete an object earlier will be denied.
    uniform_bucket_level_access (bool|None):
      Determines if the IAM policies will apply to every object in bucket.
  """

  def __init__(self,
               cors_file_path=None,
               default_encryption_key=None,
               default_event_based_hold=None,
               default_storage_class=None,
               labels_file_path=None,
               labels_to_append=None,
               labels_to_remove=None,
               lifecycle_file_path=None,
               location=None,
               log_bucket=None,
               log_object_prefix=None,
               retention_period=None,
               requester_pays=None,
               uniform_bucket_level_access=None,
               versioning=None,
               web_error_page=None,
               web_main_page_suffix=None):
    super(_GcsBucketConfig,
          self).__init__(cors_file_path, labels_file_path, labels_to_append,
                         labels_to_remove, lifecycle_file_path, location,
                         log_bucket, log_object_prefix, requester_pays,
                         versioning, web_error_page, web_main_page_suffix)
    self.default_encryption_key = default_encryption_key
    self.default_event_based_hold = default_event_based_hold
    self.default_storage_class = default_storage_class
    self.requester_pays = requester_pays
    self.retention_period = retention_period
    self.uniform_bucket_level_access = uniform_bucket_level_access

  def __eq__(self, other):
    return (super(_GcsBucketConfig, self).__eq__(other) and
            self.default_encryption_key == other.default_encryption_key and
            self.default_event_based_hold == other.default_event_based_hold and
            self.default_storage_class == other.default_storage_class and
            self.requester_pays == other.requester_pays and
            self.retention_period == other.retention_period and
            self.uniform_bucket_level_access
            == other.uniform_bucket_level_access)


class _S3BucketConfig(_BucketConfig):
  """Holder for S3-specific bucket fields.

  See superclass for attributes.
  We currently don't support any S3-only fields. This class exists to maintain
  the provider-specific subclass pattern used by the request config factory.
  """


class _ObjectConfig(object):
  """Holder for storage object settings shared between cloud providers.

  Provider-specific subclasses may add more attributes.

  Attributes:
    cache_control (str|None): Influences how backend caches requests and
      responses.
    content_disposition (str|None): Information on how content should be
      displayed.
    content_encoding (str|None): How content is encoded (e.g. "gzip").
    content_language (str|None): Content's language (e.g. "en" = "English).
    content_type (str|None): Type of data contained in content (e.g.
      "text/html").
    custom_metadata (dict|None): Custom metadata fields set by user.
    decryption_key (encryption_util.EncryptionKey): The key that should be used
      to decrypt information in GCS.
    encryption_key (encryption_util.EncryptionKey): The key that should be used
      to encrypt information in GCS.
    md5_hash (str|None): MD5 digest to use for validation.
    size (int|None): Object size in bytes.
    storage_class (str|None): Storage class for cloud object. If None, will use
      bucket's default.
  """

  def __init__(self,
               cache_control=None,
               content_disposition=None,
               content_encoding=None,
               content_language=None,
               content_type=None,
               custom_metadata=None,
               decryption_key=None,
               encryption_key=None,
               md5_hash=None,
               size=None,
               storage_class=None):
    self.cache_control = cache_control
    self.content_disposition = content_disposition
    self.content_encoding = content_encoding
    self.content_language = content_language
    self.content_type = content_type
    self.custom_metadata = custom_metadata
    self.decryption_key = decryption_key
    self.encryption_key = encryption_key
    self.md5_hash = md5_hash
    self.size = size
    self.storage_class = storage_class

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (self.cache_control == other.cache_control and
            self.content_disposition == other.content_disposition and
            self.content_encoding == other.content_encoding and
            self.content_language == other.content_language and
            self.content_type == other.content_type and
            self.custom_metadata == other.custom_metadata and
            self.decryption_key == other.decryption_key and
            self.encryption_key == other.encryption_key and
            self.md5_hash == other.md5_hash and self.size == other.size and
            self.storage_class == other.storage_class)

  def __repr__(self):
    return debug_output.generic_repr(self)


class _GcsObjectConfig(_ObjectConfig):
  """Arguments object for requests with custom GCS parameters.

  See super class for additional attributes.

  Attributes:
    custom_time (datetime|None): Custom time user can set.
  """
  # pylint:enable=g-missing-from-attributes

  def __init__(self,
               cache_control=None,
               content_disposition=None,
               content_encoding=None,
               content_language=None,
               content_type=None,
               custom_metadata=None,
               custom_time=None,
               decryption_key=None,
               encryption_key=None,
               md5_hash=None,
               size=None):
    super(_GcsObjectConfig, self).__init__(
        cache_control=cache_control,
        content_disposition=content_disposition,
        content_encoding=content_encoding,
        content_language=content_language,
        content_type=content_type,
        custom_metadata=custom_metadata,
        decryption_key=decryption_key,
        encryption_key=encryption_key,
        md5_hash=md5_hash,
        size=size)
    self.custom_time = custom_time

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (super(_GcsObjectConfig, self).__eq__(other) and
            self.custom_time == other.custom_time)


class _S3ObjectConfig(_ObjectConfig):
  """We currently do not support any S3-specific object configurations."""


class _RequestConfig(object):
  """Holder for parameters shared between cloud providers.

  Provider-specific subclasses may add more attributes.

  Attributes:
    predefined_acl_string (str|None): ACL to set on resource.
    predefined_default_acl_string (str|None): Default ACL to set on resources.
    resource_args (_BucketConfig|_ObjectConfig|None): Holds settings for a cloud
      resource.
  """

  def __init__(self,
               predefined_acl_string=None,
               predefined_default_acl_string=None,
               resource_args=None):
    self.predefined_acl_string = predefined_acl_string
    self.predefined_default_acl_string = predefined_default_acl_string
    self.resource_args = resource_args

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (self.predefined_acl_string == other.predefined_acl_string and
            self.predefined_default_acl_string
            == other.predefined_default_acl_string and
            self.resource_args == other.resource_args)

  def __repr__(self):
    return debug_output.generic_repr(self)


# pylint:disable=g-missing-from-attributes
class _GcsRequestConfig(_RequestConfig):
  """Holder for GCS-specific API request parameters.

  See super class for additional attributes.

  Attributes:
    gzip_in_flight (str|list|None): Whether to use gzip transport encoding for
      uploading files. Can be string constant saying to encode all files, list
      saying to encode files with specific extensions, or None saying not to
      encode any files.
    max_bytes_per_call (int|None): Integer describing maximum number of bytes to
      write per service call.
    no_clobber (bool): Do not copy if destination resource already exists.
    precondition_generation_match (int|None): Perform request only if generation
      of target object matches the given integer. Ignored for bucket requests.
    precondition_metageneration_match (int|None): Perform request only if
      metageneration of target object/bucket matches the given integer.
  """
  # pylint:enable=g-missing-from-attributes

  def __init__(self,
               gzip_in_flight=None,
               max_bytes_per_call=None,
               no_clobber=False,
               precondition_generation_match=None,
               precondition_metageneration_match=None,
               predefined_acl_string=None,
               predefined_default_acl_string=None,
               resource_args=None):
    super(_GcsRequestConfig, self).__init__(
        predefined_acl_string=predefined_acl_string,
        predefined_default_acl_string=predefined_default_acl_string,
        resource_args=resource_args)
    self.gzip_in_flight = gzip_in_flight
    self.max_bytes_per_call = max_bytes_per_call
    self.no_clobber = no_clobber
    self.precondition_generation_match = precondition_generation_match
    self.precondition_metageneration_match = precondition_metageneration_match

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (super(_GcsRequestConfig, self).__eq__(other) and
            self.gzip_in_flight == other.gzip_in_flight and
            self.max_bytes_per_call == other.max_bytes_per_call and
            self.no_clobber == other.no_clobber and
            self.precondition_generation_match
            == other.precondition_generation_match and
            self.precondition_metageneration_match
            == other.precondition_metageneration_match)


class _S3RequestConfig(_RequestConfig):
  """Holder for S3-specific API request parameters.

  Currently just meant for use with S3ObjectConfig and S3BucketConfig in
  the parent class "resource_args" field.
  """


def _get_request_config_resource_args(url,
                                      content_type=None,
                                      decryption_key_hash=None,
                                      md5_hash=None,
                                      size=None,
                                      user_request_args=None):
  """Generates metadata for API calls to storage buckets and objects."""
  if not isinstance(url, storage_url.CloudUrl):
    return None
  user_resource_args = getattr(user_request_args, 'resource_args', None)
  new_resource_args = None

  if url.is_bucket():
    if url.scheme in storage_url.VALID_CLOUD_SCHEMES:
      if url.scheme == storage_url.ProviderPrefix.GCS:
        new_resource_args = _GcsBucketConfig()
        if user_resource_args:
          new_resource_args.default_encryption_key = (
              user_resource_args.default_encryption_key)
          new_resource_args.default_event_based_hold = (
              user_resource_args.default_event_based_hold)
          new_resource_args.default_storage_class = (
              user_resource_args.default_storage_class)
          new_resource_args.retention_period = (
              user_resource_args.retention_period)
          new_resource_args.uniform_bucket_level_access = (
              user_resource_args.uniform_bucket_level_access)

      elif url.scheme == storage_url.ProviderPrefix.S3:
        new_resource_args = _S3BucketConfig()

        error_flags_present = []
        for field in S3_ERROR_FLAGS:
          if getattr(user_request_args, field, None):
            for flag in S3_ERROR_FLAGS[field]:
              error_flags_present.append(flag)
        if error_flags_present:
          raise ValueError('Flags disallowed for S3: {}'.format(
              ', '.join(error_flags_present)))

        warning_flags_present = []
        for field, flag in S3_WARNING_FLAGS:
          if getattr(user_resource_args, field, None):
            warning_flags_present.append(flag)
        if warning_flags_present:
          log.warning('Some flags do not have S3 support: {}.'.format(
              ', '.join(warning_flags_present)))

    else:
      new_resource_args = _BucketConfig()

    new_resource_args.location = getattr(user_resource_args, 'location', None)
    new_resource_args.cors_file_path = getattr(
        user_resource_args, 'cors_file_path', None)
    new_resource_args.labels_file_path = getattr(
        user_resource_args, 'labels_file_path', None)
    new_resource_args.labels_to_append = getattr(
        user_resource_args, 'labels_to_append', None)
    new_resource_args.labels_to_remove = getattr(
        user_resource_args, 'labels_to_remove', None)
    new_resource_args.lifecycle_file_path = getattr(
        user_resource_args, 'lifecycle_file_path', None)
    new_resource_args.log_bucket = getattr(
        user_resource_args, 'log_bucket', None)
    new_resource_args.log_object_prefix = getattr(
        user_resource_args, 'log_object_prefix', None)
    new_resource_args.requester_pays = getattr(user_resource_args,
                                               'requester_pays', None)
    new_resource_args.versioning = getattr(
        user_resource_args, 'versioning', None)
    new_resource_args.web_error_page = getattr(
        user_resource_args, 'web_error_page', None)
    new_resource_args.web_main_page_suffix = getattr(
        user_resource_args, 'web_main_page_suffix', None)

  elif url.is_object():
    if url.scheme == storage_url.ProviderPrefix.GCS:
      new_resource_args = _GcsObjectConfig()
      if user_resource_args:
        new_resource_args.custom_time = user_resource_args.custom_time

    elif url.scheme == storage_url.ProviderPrefix.S3:
      new_resource_args = _S3ObjectConfig()

    else:
      new_resource_args = _ObjectConfig()

    new_resource_args.content_type = content_type
    new_resource_args.md5_hash = md5_hash
    new_resource_args.size = size

    new_resource_args.encryption_key = encryption_util.get_encryption_key()
    if decryption_key_hash:
      new_resource_args.decryption_key = encryption_util.get_decryption_key(
          decryption_key_hash)

    if user_resource_args:
      # User args should override existing settings.
      if user_resource_args.content_type is not None:
        if user_resource_args.content_type:
          new_resource_args.content_type = user_resource_args.content_type
        else:  # Empty string or other falsey value but not completely unset.
          new_resource_args.content_type = DEFAULT_CONTENT_TYPE

      if user_resource_args.md5_hash is not None:
        new_resource_args.md5_hash = user_resource_args.md5_hash

      new_resource_args.cache_control = user_resource_args.cache_control
      new_resource_args.content_disposition = user_resource_args.content_disposition
      new_resource_args.content_encoding = user_resource_args.content_encoding
      new_resource_args.content_language = user_resource_args.content_language
      new_resource_args.custom_metadata = user_resource_args.custom_metadata
      if user_resource_args.storage_class:
        # Currently, all providers require all caps storage classes.
        new_resource_args.storage_class = (
            user_resource_args.storage_class.upper())

  return new_resource_args


def get_request_config(url,
                       content_type=None,
                       decryption_key_hash=None,
                       md5_hash=None,
                       size=None,
                       user_request_args=None):
  """Generates API-specific RequestConfig. See output classes for arg info."""
  resource_args = _get_request_config_resource_args(url, content_type,
                                                    decryption_key_hash,
                                                    md5_hash, size,
                                                    user_request_args)

  if url.scheme == storage_url.ProviderPrefix.GCS:
    request_config = _GcsRequestConfig(resource_args=resource_args)
    if user_request_args:
      request_config.gzip_in_flight = user_request_args.gzip_in_flight
      if user_request_args.max_bytes_per_call:
        request_config.max_bytes_per_call = int(
            user_request_args.max_bytes_per_call)
      if user_request_args.no_clobber:
        request_config.no_clobber = user_request_args.no_clobber
      if user_request_args.precondition_generation_match:
        request_config.precondition_generation_match = int(
            user_request_args.precondition_generation_match)
      if user_request_args.precondition_metageneration_match:
        request_config.precondition_metageneration_match = int(
            user_request_args.precondition_metageneration_match)
  elif url.scheme == storage_url.ProviderPrefix.S3:
    request_config = _S3RequestConfig(resource_args=resource_args)
  else:
    request_config = _RequestConfig(resource_args=resource_args)

  request_config.predefined_acl_string = getattr(user_request_args,
                                                 'predefined_acl_string', None)
  request_config.predefined_default_acl_string = getattr(
      user_request_args, 'predefined_default_acl_string', None)

  return request_config



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting a service."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.events import eventflow_operations
from googlecloudsdk.command_lib.events import exceptions
from googlecloudsdk.command_lib.events import resource_args
from googlecloudsdk.command_lib.events import util
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import flags as serverless_flags
from googlecloudsdk.command_lib.run import platforms
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import console_io


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class Delete(base.Command):
  """Delete a trigger."""

  detailed_help = {
      'DESCRIPTION': """\
          {description}
          """,
      'EXAMPLES': """\
          To delete a trigger:

              $ {command} TRIGGER
          """,
  }

  @staticmethod
  def CommonArgs(parser):
    """Defines arguments common to all release tracks."""
    trigger_presentation = presentation_specs.ResourcePresentationSpec(
        'trigger',
        resource_args.GetTriggerResourceSpec(),
        'Name of the trigger to delete',
        required=True)
    concept_parsers.ConceptParser([trigger_presentation]).AddToParser(parser)

  @staticmethod
  def Args(parser):
    Delete.CommonArgs(parser)

  def Run(self, args):
    """Executes when the user runs the delete command."""
    conn_context = connection_context.GetConnectionContext(
        args, serverless_flags.Product.EVENTS, self.ReleaseTrack())

    trigger_ref = args.CONCEPTS.trigger.Parse()
    console_io.PromptContinue(
        message='Trigger [{}] will be deleted.'.format(trigger_ref.Name()),
        throw_if_unattended=True,
        cancel_on_no=True)

    with eventflow_operations.Connect(conn_context) as client:
      # TODO(b/147308604): Don't delete source when Odin supports ownerRefs
      if platforms.GetPlatform() == platforms.PLATFORM_MANAGED:
        trigger_obj = client.GetTrigger(trigger_ref)
        if trigger_obj is not None:
          source_crds = client.ListSourceCustomResourceDefinitions()
          source_ref, source_crd = util.GetSourceRefAndCrdForTrigger(
              trigger_obj, source_crds, client.IsCluster())
          if source_ref and source_crd:
            # Delete the source before the trigger because we need the trigger
            # to exist to be able to find the source. Otherwise, we could end up
            # losing a reference to the source if trigger deletion succeeds but
            # source deletion fails.
            try:
              client.DeleteSource(source_ref, source_crd)
            except exceptions.SourceNotFound:
              # Source could have been deleted but trigger deletion failed
              # and this command was re-run, which is fine.
              pass
      if client.IsCluster():
        trigger_ref = resources.REGISTRY.Parse(
            trigger_ref.RelativeName(),
            collection=util.ANTHOS_TRIGGER_COLLECTION_NAME,
            api_version=client.api_version)
      client.DeleteTrigger(trigger_ref)

    log.DeletedResource(trigger_ref.Name(), 'trigger')



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for spanner databases set-iam-policy."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.spanner import iam
from googlecloudsdk.command_lib.spanner import resource_args


class SetIamPolicy(base.Command):
  """Set the IAM policy for a Cloud Spanner database."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
      The following command reads an IAM policy defined in a JSON file
      `policy.json` and sets it for a spanner database with the ID
      `my-database-id`:

        $ {command} my-database-id --instance=my-instance-id policy.json

      See https://cloud.google.com/iam/docs/managing-policies for details of the
      policy file format and contents.
        """),
  }

  @staticmethod
  def Args(parser):
    """See base class."""
    resource_args.AddDatabaseResourceArg(parser,
                                         'to set IAM policy binding for')
    parser.add_argument(
        'policy_file', help='Name of JSON or YAML file with the IAM policy.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    database_ref = args.CONCEPTS.database.Parse()
    result = iam.SetDatabaseIamPolicy(database_ref, args.policy_file)
    iam_util.LogSetIamPolicy(database_ref.Name(), 'database')
    return result



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Cloud Pub/Sub subscription describe command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.pubsub import subscriptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.pubsub import resource_args


class Describe(base.DescribeCommand):
  """Describes a Cloud Pub/Sub subscription."""

  @staticmethod
  def Args(parser):
    resource_args.AddSubscriptionResourceArg(parser, 'to describe.')

  def Run(self, args):
    client = subscriptions.SubscriptionsClient()
    subscription_ref = args.CONCEPTS.subscription.Parse()

    return client.Get(subscription_ref)
